{----------------------------------------------------}
{ Andreis LCARS-Components 1.2 for Delphi 2, (3), 4 }
{----------------------------------------------------}
{ Copyright (c) 1999 Andrei Scheibner                }
{ http://www.htwm.de/~ascheibn                       }
{ ascheibn@htwm.de                                   }
{----------------------------------------------------}
{These Components are Freeware. You can distribute them}
{altogether, you can improve them , but I would appreciate}
{it, if my name appears somewhere in your improvements.}

unit LCARSAllocator;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;


type
  TMarginShape = (msHalfRound, msRectangle, msRounded);
  TButtonShape = (bsRectangle, bsRounded);
  TMarginStyle = (msDivided, msFull);
  TButtonInfo = record
                 XLow : Integer;
                 XHigh : Integer;
                 YLow : Integer;
                 YHigh : Integer;
                 Used : Boolean;
                end;
  TColArray = Array[0..9] of Integer;
  TLCARSAllocator = class(TGraphicControl)
  private
   FBack : TBitmap;
   FBackColor : TColor;
   FEnabledColor : TColor;
   FDisabledColor : TColor;
   FMarginColor : TColor;
   FDummyColor : TColor;
   FCaptionWidth : integer;
   FCaptionHeight : integer;
   FCaptionPosX : integer;
   FCaptionPosY : integer;
   FValueWidth : integer;
   FValueHeight : integer;
   FValuePosX : integer;
   FValuePosY : integer;
   FValueAsString : String;
   FValueOnButtons : Boolean;
   FDummyWidth : integer;
   FDummyHeight : integer;
   FDummyPosX : integer;
   FDummyPosY : integer;
   FDummyDiffX : Integer;
   FDummyDiffY : Integer;
   FShowDummy : Boolean;
   FTransferInProgress : Boolean;
   FItems: TStrings;
   FMaxButtonsPerLine : Integer;
   FValuePerButton : Integer;
   {FItemIndex: Integer;}
   FPosXLow : Integer;
   FPosXHigh : Integer;
   FPosYLow : Integer;
   FPosYHigh : Integer;
   FButtonShape : TButtonShape;
   FMarginShape : TMarginShape;
   FMarginStyle : TMarginStyle;
   FMarginRadius : Integer;
   FButtonPlace : Integer;
   FMarginWidth : Integer;
   FCurrentColor : TColor;
   FButtonHeight : Integer;
   FButtonWidth : Integer;
   FDIvHeightRemain : Integer;
   FDivWidthRemain : Integer;
   FButtonArray : Array[0..9,0..19] of TButtonInfo;
   FButtons : TColArray;
   FValues : TColArray;
   FMouseSource : Integer;
   FMouseDest : Integer;
   FMouseInControl: Boolean;
   FOnMouseEnter: TNotifyEvent;
   FOnMouseLeave: TNotifyEvent;
   FOnChange : TNotifyEvent;
   procedure SetItems(Value: TStrings);
   procedure SetEnabledColor(Value : TColor);
   procedure SetDisabledColor(Value : TColor);
   procedure SetMarginColor(Value : TColor);
   procedure SetButtonShape(Value : TButtonShape);
   procedure SetMarginShape(Value : TMarginShape);
   procedure SetMarginStyle(Value : TMarginStyle);
   procedure SetButtonPlace(Value : Integer);
   procedure SetMarginRadius(Value : Integer);
   procedure SetMaxButtonsPerLine(Value : Integer);
   procedure SetValuePerButton(Value : Integer);
   procedure SetValueOnButtons(Value : Boolean);
   procedure SetShowDummy(Value : Boolean);
   procedure SetDummyColor(Value : TColor);
   procedure FillButtonArray;
   procedure ClearButtonArray;
   procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
   procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
   procedure Paint; override;
   procedure Change ; dynamic;
   procedure MouseEnter; dynamic;
   procedure MouseLeave; dynamic;
   procedure MouseDown(Button:TMouseButton;
                       Shift:TShiftState;X,Y:Integer);override;
   procedure MouseMove(Shift:TShiftState;
                       X,Y:Integer);override;
   procedure MouseUp(Button:TMouseButton;
                     Shift:TShiftState;X,Y:Integer);override;
  public
   property MouseInControl: Boolean read FMouseInControl;
   property Buttons : TColArray read FButtons;
   property Values : TColArray read FValues;
   constructor Create(AOwner : TComponent) ; override;
   destructor Destroy; override;
   function Transfer(Source : Integer; Dest : Integer; ButtonAmount : Integer) : Boolean;
   function TransferAll(Source : Integer; Dest : Integer) : Boolean;
  published
   property Parent;
   property Width default 129;
   property Height default 41;
   property Visible default true;
   property Enabled default true;
   property Font;
   property ParentFont default true;
   property Hint;
   property ShowHint;
   property ParentShowHint;
   property Align;
   property Items: TStrings read FItems write SetItems;
   property EnabledColor : TColor read FEnabledColor
    write SetEnabledColor default clgreen;
   property DisabledColor : TColor read FDisabledColor
    write SetDisabledColor default clgray;
   property MarginColor : TColor read FMarginColor
    write SetMarginColor default clblue;
   property ButtonShape : TButtonShape read FButtonShape write SetButtonShape;
   property MarginShape : TMarginShape read FMarginShape write SetMarginShape;
   property MarginStyle : TMarginStyle read FMarginStyle write SetMarginStyle;
   property ButtonPlace : Integer read FButtonPlace write SetButtonPlace;
   property MarginRadius : Integer read FMarginRadius write SetMarginRadius;
   property MaxButtonsPerLine : Integer read FMaxButtonsPerLine
    write SetMaxButtonsPerLine;
   property ValueOnButtons : Boolean read FValueOnButtons write SetValueOnButtons;
   property ValuePerButton : Integer read FValuePerButton write SetValuePerButton;
   property ShowDummy : Boolean read FShowDummy write SetShowDummy;
   property DummyColor : TColor read FDummyColor write SetDummyColor;
   property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
   property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
   property OnChange: TNotifyEvent read FOnChange write FOnChange;
   property OnClick;
   property OnDblClick;
   property OnMouseDown;
   property OnMouseUp;
   property OnMouseMove;
   property OnDragDrop;
   property OnDragOver;
   property OnEndDrag;
  end;

implementation

constructor TLCARSAllocator.Create(AOwner : TComponent);
begin
 inherited Create(AOwner);
 Width := 200;
 Height := 100;
 ButtonPlace := Width - 50;
 FBackColor := clBlack;
 FEnabledColor := clGreen;
 FDisabledColor := clSilver;
 FMarginColor := clYellow;
 FDummyColor := FEnabledColor;
 Visible := true;
 ControlStyle:=ControlStyle+[csOpaque];
 FItems := TStringList.Create;
 FMaxButtonsPerLine := 10;
 FButtonShape := bsRounded;
 FMarginShape := msRounded;
 FMarginStyle := msDivided;
 FMarginRadius := 20;
 FMaxButtonsPerLine := 10;
 FValuePerButton := 10;
 FValueOnButtons := true;
 FShowDummy := false;
 FButtons[0] := 10;
 FBack := TBitmap.Create;
 ParentFont := true;
end;

destructor TLCARSAllocator.Destroy;
begin
 FBack.Free;
 FItems.Free;
 inherited Destroy;
end;

Procedure TLCARSAllocator.MouseDown(Button:TMouseButton;
                                    Shift:TShiftState;X,Y:Integer);
var FListCounter : Integer;
begin
 if Items.Count > 0 then
  begin
   for FListCounter := 0 to FItems.Count - 1 do
    begin
     if (y > FButtonArray[FListCounter][0].YLow) and
        (y < FButtonArray[FListCounter][0].YHigh) then
      begin
       FMouseSource := FListCounter;
       FDummyPosY := FButtonArray[FListCounter][0].YLow;
       FDummyHeight := FButtonArray[FListCounter][0].YHigh -
                       FButtonArray[FListCounter][0].YLow;
       FDummyDiffY := y - FDummyPosY;
       break;
      end;
    end;
   if FShowDummy = true then
    begin
     for FListCounter := 0 to FMaxButtonsPerLine do
      begin
       if (x > FButtonArray[0][FListCounter].XLow) and
          (x < FButtonArray[0][FListCounter].XHigh) then
        begin
         FDummyPosX := FButtonArray[0][FListCounter].XLow;
         FDummyWidth := FButtonArray[0][FListCounter].XHigh -
                         FButtonArray[0][FListCounter].XLow;
         FDummyDiffX := x - FDummyPosX;
         break;
        end;
      end;
     FTransferInProgress := true;
     if FButtons[FMouseSource] = 0 then
      FTransferInProgress := false;
    end;
  end;
 inherited MouseDown(Button,Shift,X,Y);
end;

Procedure TLCARSAllocator.MouseMove(Shift:TShiftState;X,Y:Integer);
begin
 if FTransferInProgress = true then
  begin
   FDummyPosX := x - FDummyDiffX;
   FDummyPosY := y - FDummyDiffY;
   Repaint;
  end;
 inherited MouseMove(Shift, x, y);
end;

Procedure TLCARSAllocator.MouseUp(Button:TMouseButton;
                                   Shift:TShiftState;X,Y:Integer);
var FListCounter : Integer;
begin
 FTransferInProgress := false;
 if Items.Count > 0 then
  begin
   for FListCounter := 0 to Items.Count - 1 do
    begin
     if (y > FButtonArray[FListCounter][0].YLow) and
        (y < FButtonArray[FListCounter][0].YHigh) then
      begin
       FMouseDest := FListCounter;
       break;
      end;
    end;
   Transfer(FMouseSource, FMouseDest, 1);
  end;
 inherited MouseUp(Button,Shift,X,Y);
end;

procedure TLCARSAllocator.ClearButtonArray;
var FArrayCounterV, FArrayCounterH : Integer;
begin
 for FArrayCounterV := 0 to 9 do
  for FArrayCounterH := 0 to 19 do
   begin
    FButtonArray[FArrayCounterV][FArrayCounterH].YLow := 0;
    FButtonArray[FArrayCounterV][FArrayCounterH].YHigh := 0;
    FButtonArray[FArrayCounterV][FArrayCounterH].XLow := 0;
    FButtonArray[FArrayCounterV][FArrayCounterH].XHigh := 0;
    FButtonArray[FArrayCounterV][FArrayCounterH].Used := false;
   end;
end;

procedure TLCARSAllocator.FillButtonArray;
var FArrayCounterV, FArrayCounterH : Integer;
begin
 if Items.Count > 0 then
  begin
   FPosYLow := -2;
   FPosYHigh := -2;
   FButtonHeight := (FBack.Height + 4) div Items.Count;
   FDivHeightRemain := (FBack.Height + 4) mod Items.Count;
   for FArrayCounterV := 0 to (Items.Count - 1) do
    begin
     FPosYLow := FPosYHigh;
     FPosYHigh := FPosYHigh + FButtonHeight;
     if FDivHeightRemain > 0 then
      begin
       Inc(FPosYHigh);
       Dec(FDivHeightRemain);
      end;
     FPosXLow := FMarginWidth;
     FPosXHigh := FMarginWidth;
     FButtonWidth := FButtonPlace div FMaxButtonsPerLine;
     FDivWidthRemain := FButtonPlace mod FMaxButtonsPerLine;
     for FArrayCounterH := 0 to FMaxButtonsPerLine do
      begin
       FPosXLow := FPosXHigh;
       FPosXHigh := FPosXHigh + FButtonWidth;
       if FDivWidthRemain > 0 then
        begin
         Inc(FPosXHigh);
         Dec(FDivWidthRemain);
        end;
       FButtonArray[FArrayCounterV][FArrayCounterH].YLow := FPosYLow;
       FButtonArray[FArrayCounterV][FArrayCounterH].YHigh := FPosYHigh;
       FButtonArray[FArrayCounterV][FArrayCounterH].XLow := FPosXLow;
       FButtonArray[FArrayCounterV][FArrayCounterH].XHigh := FPosXHigh;
       if FButtons[FArrayCounterV] > FArrayCounterH then
        FButtonArray[FArrayCounterV][FArrayCounterH].Used := true;
      end;
     FValues[FArrayCounterV] := FButtons[FArrayCounterV] * FValuePerButton;
    end;
  end;
end;

procedure TLCARSAllocator.Paint;
var VisibleRect, BackRect : TRect;
    FListCounter : Integer;
    FArrayCounterV, FArrayCounterH : Integer;
begin
 FBack.Canvas.Font.Assign(Font);
 FBack.Width := Width;
 FBack.Height := Height;
 if Parent <> nil then
  with Parent do
   FBackColor := Color;
 if Enabled = true then
  FCurrentColor := FEnabledColor
 else
  FCurrentColor := FDisabledColor;
 if (FButtonPlace + FMarginWidth) <> Width then
  begin
   FButtonPlace := FButtonPlace + (Width - FButtonPlace - FMarginWidth);
   FMarginWidth := Width - FButtonPlace;
  end;
 FBack.Canvas.Brush.Color := FBackColor;
 FBack.Canvas.Pen.Color := FBackColor;
 FBack.Canvas.Rectangle(0, 0, FBack.Width, FBack.Height);
 if Items.Count > 0 then
  begin
   FCaptionWidth :=  0;
   for FListCounter := 0 to Items.Count - 1 do
    begin
     if FBack.Canvas.TextWidth(Items[FListCounter]) > FCaptionWidth then
      FCaptionWidth := FBack.Canvas.TextWidth(Items[FListCounter]);
    end;
   ClearButtonArray;
   FillButtonArray;
   FBack.Canvas.Brush.Color := FMarginColor;
   FBack.Canvas.Pen.Color := FMarginColor;
   if FMarginStyle = msFull then
    begin
     if FMarginShape = msHalfRound then
      begin
       FBack.Canvas.RoundRect(0, 0, FBack.Width, FBack.Height, FMarginRadius, FMarginRadius);
       FBack.Canvas.Brush.Color := FBackColor;
       FBack.Canvas.Pen.Color := FBackColor;
       FBack.Canvas.Rectangle(FMarginWidth - 2, 0, FBack.Width, FBack.Height);
       FBack.Canvas.Brush.Color := FMarginColor;
       FBack.Canvas.Pen.Color := FMarginColor;
      end
     else if FMarginShape = msRectangle then
      begin
       FBack.Canvas.Rectangle(0, 0, FMarginWidth - 2, FBack.Height);
      end
     else if FMarginShape = msRounded then
      begin
        FBack.Canvas.RoundRect(0, 0, FMarginWidth - 2, FBack.Height,
                               FMarginRadius, FMarginRadius);
      end;
    end
   else if FMarginStyle = msDivided then
    begin
     for FlistCounter := 0 to Items.Count - 1 do
      begin
       if FMarginShape = msHalfRound then
        begin
         FBack.Canvas.Ellipse(0, FButtonArray[FListCounter][0].YLow + 2,
                              FButtonHeight - 4, FButtonArray[FListCounter][0].YHigh - 2);
         FBack.Canvas.Rectangle(((FButtonHeight - 4) div 2), FButtonArray[FListCounter][0].YLow + 2,
                                FMarginWidth - 2, FButtonArray[FListCounter][0].YHigh - 2);
        end
       else if FMarginShape = msRectangle then
        begin
         FBack.Canvas.Rectangle(0, FButtonArray[FListCounter][0].YLow + 2,
                                FMarginWidth - 2, FButtonArray[FListCounter][0].YHigh - 2);
        end
       else if FMarginShape = msRounded then
        begin
         FBack.Canvas.RoundRect(0, FButtonArray[FListCounter][0].YLow + 2,
                                FMarginWidth -2, FButtonArray[FListCounter][0].YHigh - 2,
                                FButtonHeight - 4, FButtonHeight - 4);
        end;
      end;
    end;
   FCaptionWidth :=  0;
   if FMarginWidth > 0 then
    begin
     for FListCounter := 0 to Items.Count - 1 do
      begin
       if FBack.Canvas.TextWidth(Items[FListCounter]) > FCaptionWidth then
        FCaptionWidth := FBack.Canvas.TextWidth(Items[FListCounter]);
      end;
     for FListCounter := 0 to Items.Count - 1 do
      begin
       FCaptionHeight := FBack.Canvas.TextHeight(Items[FListCounter]);
       FCaptionPosY := ((FButtonArray[FListCounter][0].YLow + FButtonArray[FListCounter][0].YHigh) div 2) - (FCaptionHeight div 2);
       FCaptionPosX := (FMarginWidth div 2) - (FCaptionWidth div 2) - 1;
       FBack.Canvas.TextOut(FCaptionPosX, FCaptionPosY, Items[FListCounter]);
      end;
    end;
   FBack.Canvas.Brush.Color := FCurrentColor;
   FBack.Canvas.Pen.Color := FCurrentColor;
   for FArrayCounterV := 0 to (Items.Count - 1) do
    for FArrayCounterH := 0 to FMaxButtonsPerLine do
     begin
      if FButtonArray[FArrayCounterV][FArrayCounterH].Used = true then
       begin
        if FButtonShape = bsRectangle then
         begin
          FBack.Canvas.Rectangle(FButtonArray[FArrayCounterV][FArrayCounterH].XLow + 2,
                                 FButtonArray[FArrayCounterV][FArrayCounterH].YLow + 2,
                                 FButtonArray[FArrayCounterV][FArrayCounterH].XHigh - 2,
                                 FButtonArray[FArrayCounterV][FArrayCounterH].YHigh - 2);
         end
        else if FButtonShape = bsRounded then
         begin
          FBack.Canvas.RoundRect(FButtonArray[FArrayCounterV][FArrayCounterH].XLow + 2,
                                 FButtonArray[FArrayCounterV][FArrayCounterH].YLow + 2,
                                 FButtonArray[FArrayCounterV][FArrayCounterH].XHigh - 2,
                                 FButtonArray[FArrayCounterV][FArrayCounterH].YHigh - 2,
                                 FButtonHeight - 4, FButtonHeight - 4);
         end;
        if FValueOnButtons = true then
         begin
          FValueAsString := InttoStr((FArrayCounterH + 1) * FValuePerButton);
          FValueWidth := FBack.Canvas.TextWidth(FValueAsString);
          FValueHeight := FBack.Canvas.TextHeight(FValueAsString);
          FValuePosX := ((FButtonArray[FArrayCounterV][FArrayCounterH].XLow + FButtonArray[FArrayCounterV][FArrayCounterH].XHigh) div 2)
                        - (FValueWidth div 2);
          FValuePosY := ((FButtonArray[FArrayCounterV][FArrayCounterH].YLow + FButtonArray[FArrayCounterV][FArrayCounterH].YHigh) div 2)
                        - (FValueHeight div 2);
          FBack.Canvas.TextOut(FValuePosX, FValuePosY, FValueAsString);
         end;
       end;
     end;
   if FTransferInProgress = true then
    begin
      FBack.Canvas.Brush.Color := FDummyColor;
      FBack.Canvas.Pen.Color := FDummyColor;
      if FButtonShape = bsRectangle then
      begin
       FBack.Canvas.Rectangle(FDummyPosX + 2, FDummyPosY + 2,
                              FDummyPosX + FDummyWidth - 2, FDummyPosY + FDummyHeight - 2);
      end
     else if FButtonShape = bsRounded then
      begin
       FBack.Canvas.RoundRect(FDummyPosX + 2, FDummyPosY + 2,
                              FDummyPosX + FDummyWidth - 2, FDummyPosY + FDummyHeight - 2,
                              FButtonHeight - 4, FButtonHeight - 4);
      end;
    end;
  end;
 VisibleRect := Rect(0, 0, Width, Height);
 BackRect := Rect(0, 0, FBack.Width, FBack.Height);
 Canvas.CopyRect(VisibleRect, FBack.Canvas, BackRect);
end;

procedure TLCARSAllocator.Change;
begin
 if Assigned(FOnChange) then FOnChange(Self);
end;

function TLCARSAllocator.Transfer(Source : Integer; Dest : Integer; ButtonAmount : Integer) : Boolean;
begin
 Transfer := false;
 if (Source < Items.Count) and (Dest < Items.Count) then
  begin
   if (FButtons[Source] >= ButtonAmount) then
    begin
     FButtons[Source] := FButtons[Source] - ButtonAmount;
     FValues[Source] := FButtons[Source] * FValuePerButton;
     FButtons[Dest] := FButtons[Dest] + ButtonAmount;
     FValues[Dest] := FButtons[Dest] * FValuePerButton;
     Change;
     Transfer := true;
     Repaint;
    end;
  end;
end;

function TLCARSAllocator.TransferAll(Source : Integer; Dest : Integer) : Boolean;
begin
 TransferAll := false;
 if (Source < Items.Count) and (Dest < Items.Count) then
  begin
   FButtons[Dest] := FButtons[Dest] + FButtons[Source];
   FButtons[Source] := 0;
   FValues[Source] := FButtons[Source] * FValuePerButton;
   FValues[Dest] := FButtons[Dest] * FValuePerButton;
   Change;
   TransferAll := true;
   Repaint;
  end;
end;

procedure TLCARSAllocator.SetDisabledColor(value : TColor);
begin
 if FDisabledColor <> value then
  begin
   FDisabledColor := value;
   Repaint;
  end;
end;

procedure TLCARSAllocator.SetEnabledColor(value : TColor);
begin
 if FEnabledColor <> value then
  begin
   FEnabledColor := value;
   Repaint;
  end;
end;

procedure TLCARSAllocator.SetMarginColor(value : TColor);
begin
 if FMarginColor <> value then
  begin
   FMarginColor := value;
   Repaint;
  end;
end;

procedure TLCARSAllocator.SetDummyColor(value : TColor);
begin
 if FDummyColor <> value then
  begin
   FDummyColor := value;
   Repaint;
  end;
end;

procedure TLCARSAllocator.SetItems(Value : TStrings);
begin
 FItems.Assign(Value);
 Repaint;
end;

procedure TLCARSAllocator.SetButtonShape(Value : TButtonShape);
begin
 if FButtonShape <> value then
  begin
   FButtonShape := value;
   Repaint;
  end;
end;

procedure TLCARSAllocator.SetMarginShape(Value : TMarginShape);
begin
 if FMarginShape <> value then
  begin
   FMarginShape := value;
   Repaint;
  end;
end;

procedure TLCARSAllocator.SetMarginStyle(Value : TMarginStyle);
begin
 if FMarginStyle <> value then
  begin
   FMarginStyle := value;
   Repaint;
  end;
end;

procedure TLCARSAllocator.SetMarginRadius(Value : Integer);
begin
 if FMarginRadius <> value then
  begin
   FMarginRadius := value;
   Repaint;
  end;
end;

procedure TLCARSAllocator.SetButtonPlace(Value : Integer);
begin
 if FButtonPlace <> Value then
  begin
   FButtonPlace := Value;
   FMarginWidth := Width - Value;
   Repaint;
  end;
end;

procedure TLCARSAllocator.SetMaxButtonsPerLine(Value : Integer);
var Difference, Counter : Integer;
begin
 Counter := 0;
 if Value > 20 then Value := 20;
 if Value < 1 then Value := 1;
 if FMaxButtonsPerLine <> Value then
  begin
   Difference := FMaxButtonsPerLine - Value;
   FMaxButtonsPerLine := Value;
   if Difference > 0 then
    begin
     repeat
      if FButtons[Counter mod Items.Count] > 0 then
       begin
        Dec(FButtons[Counter mod Items.Count]);
        Dec(Difference);
       end;
      Inc(Counter);
     until Difference = 0;
    end
   else if Difference < 0 then
    begin
     repeat
      Inc(FButtons[Counter mod Items.Count]);
      Inc(Difference);
      Inc(Counter);
     until Difference = 0;
    end;
   Repaint;
  end;
end;

procedure TLCARSAllocator.SetValuePerButton(Value : Integer);
begin
 if FValuePerButton <> Value then
  begin
   FValuePerButton := Value;
   Repaint;
  end;
end;

procedure TLCARSAllocator.SetValueOnButtons(Value : Boolean);
begin
 if FValueOnButtons <> Value then
  begin
   FValueOnButtons := Value;
   Repaint;
  end;
end;

procedure TLCARSAllocator.SetShowDummy(Value : Boolean);
begin
 if FShowDummy <> Value then
  begin
   FShowDummy := Value;
   Repaint;
  end;
end;

procedure TLCARSAllocator.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  if not FMouseInControl and Enabled {and IsForegroundTask } then begin
    FMouseInControl := True;
    MouseEnter;
  end;
end;

procedure TLCARSAllocator.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  if FMouseInControl and Enabled then begin
    FMouseInControl := False;
    MouseLeave;
  end;
end;

procedure TLCARSAllocator.MouseEnter;
begin
  if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
end;

procedure TLCARSAllocator.MouseLeave;
begin
  if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
end;

end.
