{----------------------------------------------------}
{ Andreis LCARS-Components 1.2 for Delphi 2, (3), 4 }
{----------------------------------------------------}
{ Copyright (c) 1999 Andrei Scheibner                }
{ http://www.htwm.de/~ascheibn                       }
{ ascheibn@htwm.de                                   }
{----------------------------------------------------}
{These Components are Freeware. You can distribute them}
{altogether, you can improve them , but I would appreciate}
{it, if my name appears somewhere in your improvements.}

unit LCARSRadioGroup;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;


type
  TButtonAlignment = (baLeft, baRight);
  TBMShape = (bmsHalfRound, bmsRectangle, bmsRounded);
  TMarginStyle = (msDivided, msFull);
  TLCARSRadioGroup = class(TGraphicControl)
  private
   FBack : TBitmap;
   FBackColor : TColor;
   FEnabledColor : TColor;
   FDisabledColor : TColor;
   FMarginColor : TColor;
   FMarginRadius : Integer;
   FCaptionWidth : integer;
   FCaptionHeight : integer;
   FCaptionPosX : integer;
   FCaptionPosY : integer;
   FItems: TStrings;
   FItemIndex: Integer;
   FPosLow : Integer;
   FPosHigh : Integer;
   FTempPosLow : Integer;
   FTempPosHigh : Integer;
   FButtonAlignment : TButtonAlignment;
   FButtonShape : TBMShape;
   FMarginShape : TBMShape;
   FMarginStyle : TMarginStyle;
   FButtonWidth : Integer;
   FMarginWidth : Integer;
   FCurrentColor : TColor;
   FButtonHeight : Integer;
   FBHDRemain : Integer;
   FBHDTempRemain : Integer;
   FMouseInControl: Boolean;
   FOnMouseEnter: TNotifyEvent;
   FOnMouseLeave: TNotifyEvent;
   FOnChange : TNotifyEvent;
   procedure SetItemIndex(Value: Integer);
   procedure SetItems(Value: TStrings);
   procedure SetEnabledColor(Value : TColor);
   procedure SetDisabledColor(Value : TColor);
   procedure SetMarginColor(Value : TColor);
   procedure SetButtonAlignment(Value : TButtonAlignment);
   procedure SetButtonShape(Value : TBMShape);
   procedure SetMarginShape(Value : TBMShape);
   procedure SetMarginStyle(Value : TMarginStyle);
   procedure SetButtonWidth(Value : Integer);
   procedure SetMarginRadius(Value : Integer);
   procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
   procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
    procedure Paint; override;
    procedure Change ; dynamic;
    procedure MouseEnter; dynamic;
    procedure MouseLeave; dynamic;
    procedure MouseUp(Button:TMouseButton;
                      Shift:TShiftState;X,Y:Integer);override;
  public
    property MouseInControl: Boolean read FMouseInControl;
    constructor Create(AOwner : TComponent) ; override;
    destructor Destroy; override;
  published
   property Parent;
   property Width default 129;
   property Height default 41;
   property Visible default true;
   property Enabled default true;
   property Font;
   property ParentFont default true;
   property Hint;
   property ShowHint;
   property ParentShowHint;
   property Align;
   property ItemIndex: Integer read FItemIndex write SetItemIndex default -1;
   property Items: TStrings read FItems write SetItems;
   property EnabledColor : TColor read FEnabledColor
    write SetEnabledColor default clgreen;
   property DisabledColor : TColor read FDisabledColor
    write SetDisabledColor default clgray;
   property MarginColor : TColor read FMarginColor
    write SetMarginColor default clblue;
   property ButtonAlignment : TButtonAlignment read FButtonAlignment
    write SetButtonAlignment;
   property ButtonShape : TBMShape read FButtonShape write SetButtonShape;
   property MarginShape : TBMShape read FMarginShape write SetMarginShape;
   property MarginStyle : TMarginStyle read FMarginStyle write SetMarginStyle;
   property ButtonWidth : Integer read FButtonWidth write SetButtonWidth;
   property MarginRadius : Integer read FMarginRadius write SetMarginRadius;
   property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
   property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
   property OnChange: TNotifyEvent read FOnChange write FOnChange;
   property OnClick;
   property OnDblClick;
   property OnMouseDown;
   property OnMouseUp;
   property OnMouseMove;
   property OnDragDrop;
   property OnDragOver;
   property OnEndDrag;
  end;

implementation

constructor TLCARSRadioGroup.Create(AOwner : TComponent);
begin
 inherited Create(AOwner);
 Width := 160;
 Height := 40;
 ButtonWidth := Width div 2;
 FBackColor := clBlack;
 FEnabledColor := clGreen;
 FDisabledColor := clSilver;
 FMarginColor := clYellow;
 Visible := true;
 ControlStyle:=ControlStyle+[csOpaque];
 FItems := TStringList.Create;
 FItemIndex := -1;
 FButtonAlignment := baRight;
 FButtonShape := bmsHalfRound;
 FMarginShape := bmsHalfRound;
 FMarginStyle := msDivided;
 FMarginRadius := 20;
 FBack := TBitmap.Create;
 ParentFont := true;
end;

destructor TLCARSRadioGroup.Destroy;
begin
 FBack.Free;
 FItems.Free;
 inherited Destroy;
end;

Procedure TLCARSRadioGroup.MouseUp(Button:TMouseButton;
                                   Shift:TShiftState;X,Y:Integer);
var FListCounter : Integer;
begin
 if Items.Count > 0 then
  begin
   FButtonHeight := (Height + 4) div FItems.Count;
   FBHDRemain := (Height + 4) mod FItems.Count;
   FPosLow := -2;
   FPosHigh := -2;
   for FListCounter := 0 to FItems.Count - 1 do
    begin
     FPosLow := FPosHigh;
     FPosHigh := FPosHigh + FButtonHeight;
     if FBHDRemain > 0 then
      begin
       Inc(FPosHigh);
       Dec(FBHDRemain);
      end;
     if (y > FPosLow) and (y < FPosHigh) then
      begin
       ItemIndex := FListCounter;
       break;
      end;
    end;
  end;
 inherited MouseUp(Button,Shift,X,Y);
end;

procedure TLCARSRadioGroup.Paint;
var VisibleRect, BackRect : TRect;
    FListCounter : Integer;
begin
 FBack.Canvas.Font.Assign(Font);
 FBack.Width := Width;
 FBack.Height := Height;
 if Parent <> nil then
  with Parent do
   FBackColor := Color;
 if Enabled = true then
  FCurrentColor := FEnabledColor
 else
  FCurrentColor := FDisabledColor;
 if (FButtonWidth + FMarginWidth) <> Width then
  begin
   FButtonWidth := FButtonWidth + (Width - FButtonWidth - FMarginWidth);
   FMarginWidth := Width - FButtonWidth;
  end;
 FBack.Canvas.Brush.Color := FBackColor;
 FBack.Canvas.Pen.Color := FBackColor;
 FBack.Canvas.Rectangle(0, 0, FBack.Width, FBack.Height);
 if Items.Count > 0 then
  begin
   FCaptionWidth :=  0;
   for FListCounter := 0 to Items.Count - 1 do
    begin
     if FBack.Canvas.TextWidth(Items[FListCounter]) > FCaptionWidth then
      FCaptionWidth := FBack.Canvas.TextWidth(Items[FListCounter]);
    end;
   FButtonHeight := (FBack.Height + 4) div Items.Count;
   FBHDRemain := (FBack.Height + 4) mod Items.Count;
   FPosLow := -2;
   FPosHigh := -2;
   FBack.Canvas.Brush.Color := FMarginColor;
   FBack.Canvas.Pen.Color := FMarginColor;
   if FMarginStyle = msFull then
    begin
     if FMarginShape = bmsHalfRound then
      begin
       FBack.Canvas.RoundRect(0, 0, FBack.Width, FBack.Height, FMarginRadius, FMarginRadius);
       if FButtonAlignment = baLeft then
        begin
         FBack.Canvas.Brush.Color := FBackColor;
         FBack.Canvas.Pen.Color := FBackColor;
         FBack.Canvas.Rectangle(0, 0, FButtonWidth + 2, FBack.Height);
        end
       else if FButtonAlignment = baRight then
        begin
         FBack.Canvas.Brush.Color := FBackColor;
         FBack.Canvas.Pen.Color := FBackColor;
         FBack.Canvas.Rectangle(FMarginWidth - 2, 0, FBack.Width, FBack.Height);
        end;
      end
     else if FMarginShape = bmsRectangle then
      begin
       if FButtonAlignment = baLeft then
        begin
         FBack.Canvas.Rectangle(FButtonWidth + 2, 0, FBack.Width, FBack.Height);
        end
       else if FButtonAlignment = baRight then
        begin
         FBack.Canvas.Rectangle(0, 0, FMarginWidth - 2, FBack.Height);
        end;
      end
     else if FMarginShape = bmsRounded then
      begin
       if FButtonAlignment = baLeft then
        begin
         FBack.Canvas.RoundRect(FButtonWidth + 2, 0, FBack.Width, FBack.Height,
                                FMarginRadius, FMarginRadius);
        end
       else if FButtonAlignment = baRight then
        begin
         FBack.Canvas.RoundRect(0, 0, FMarginWidth - 2, FBack.Height,
                                FMarginRadius, FMarginRadius);
        end;
      end;
    end
   else if FMarginStyle = msDivided then
    begin
     FTempPosLow := FPosLow;
     FTempPosHigh := FPosHigh;
     FBHDTempRemain := FBHDRemain;
     for FListCounter := 0 to Items.Count - 1 do
      begin
       FTempPosLow := FTempPosHigh;
       FTempPosHigh := FTempPosHigh + FButtonHeight;
       if FBHDTempRemain > 0 then
        begin
         Inc(FTempPosHigh);
         Dec(FBHDTempRemain);
        end;
       if FMarginShape = bmsHalfRound then
        begin
         if FButtonAlignment = baLeft then
          begin
           FBack.Canvas.Ellipse(FBack.Width - FButtonHeight + 4, FTempPosLow + 2,
                                FBack.Width, FTempPosHigh - 2);
           FBack.Canvas.Rectangle(FButtonWidth + 2, FTempPosLow + 2,
                                  FBack.Width - ((FButtonHeight - 4) div 2), FTempPosHigh - 2);
          end
         else if FButtonAlignment = baRight then
          begin
           FBack.Canvas.Ellipse(0, FTempPosLow + 2, FButtonHeight - 4, FTempPosHigh - 2);
           FBack.Canvas.Rectangle(((FButtonHeight - 4) div 2), FTempPosLow + 2,
                                  FMarginWidth - 2, FTempPosHigh - 2);
          end;
        end
       else if FMarginShape = bmsRectangle then
        begin
         if FButtonAlignment = baLeft then
          begin
           FBack.Canvas.Rectangle(FButtonWidth + 2, FTempPosLow + 2,
                                  FBack.Width, FTempPosHigh - 2);
          end
         else if FButtonAlignment = baRight then
          begin
           FBack.Canvas.Rectangle(0, FTempPosLow + 2,
                                  FMarginWidth - 2, FTempPosHigh - 2);
          end;
        end
       else if FMarginShape = bmsRounded then
        begin
         if FButtonAlignment = baLeft then
          begin
           FBack.Canvas.RoundRect(FButtonWidth + 2, FTempPosLow + 2,
                                  FBack.Width, FTempPosHigh - 2,
                                  FButtonHeight - 4, FButtonHeight - 4);
          end
         else if FButtonAlignment = baRight then
          begin
           FBack.Canvas.RoundRect(0, FTempPosLow + 2,
                                  FMarginWidth -2, FTempPosHigh - 2,
                                  FButtonHeight - 4, FButtonHeight - 4);
          end;
        end;
      end;
    end;
   FBack.Canvas.Brush.Color := FCurrentColor;
   FBack.Canvas.Pen.Color := FCurrentColor;
   FTempPosLow := FPosLow;
   FTempPosHigh := FPosHigh;
   FBHDTempRemain := FBHDRemain;
   for FListCounter := 0 to Items.Count - 1 do
    begin
     FTempPosLow := FTempPosHigh;
     FTempPosHigh := FTempPosHigh + FButtonHeight;
     if FBHDTempRemain > 0 then
      begin
       Inc(FTempPosHigh);
       Dec(FBHDTempRemain);
      end;
     if FListCounter = FItemIndex then
      begin
       if FButtonShape = bmsHalfRound then
        begin
         if FButtonAlignment = baLeft then
          begin
           FBack.Canvas.Ellipse(0, FTempPosLow + 2, FButtonHeight - 4, FTempPosHigh - 2);
           FBack.Canvas.Rectangle(((FButtonHeight - 4) div 2), FTempPosLow + 2,
                                  FButtonWidth - 2, FTempPosHigh - 2);
          end
         else if FButtonAlignment = baRight then
          begin
           FBack.Canvas.Ellipse(FBack.Width - FButtonHeight + 4, FTempPosLow + 2,
                                FBack.Width, FTempPosHigh - 2);
           FBack.Canvas.Rectangle(FMarginWidth + 2, FTempPosLow + 2,
                                  FBack.Width - ((FButtonHeight - 4) div 2), FTempPosHigh - 2);
          end;
        end
       else if FButtonShape = bmsRectangle then
        begin
         if FButtonAlignment = baLeft then
          begin
           FBack.Canvas.Rectangle(0, FTempPosLow + 2,
                                  FButtonWidth - 2, FTempPosHigh - 2);
          end
         else if FButtonAlignment = baRight then
          begin
           FBack.Canvas.Rectangle(FMarginWidth + 2, FTempPosLow + 2,
                                  FBack.Width, FTempPosHigh - 2);
          end;
        end
       else if FButtonShape = bmsRounded then
        begin
         if FButtonAlignment = baLeft then
          begin
           FBack.Canvas.RoundRect(0, FTempPosLow + 2,
                                  FButtonWidth -2, FTempPosHigh - 2,
                                  FButtonHeight - 4, FButtonHeight - 4);
          end
         else if FButtonAlignment = baRight then
          begin
           FBack.Canvas.RoundRect(FMarginWidth + 2, FTempPosLow + 2,
                                  FBack.Width, FTempPosHigh - 2,
                                  FButtonHeight - 4, FButtonHeight - 4);
          end;
        end;
       break;
      end;
    end;
   FTempPosLow := FPosLow;
   FTempPosHigh := FPosHigh;
   FBHDTempRemain := FBHDRemain;
   FBack.Canvas.Brush.Color := FMarginColor;
   FBack.Canvas.Pen.Color := FMarginColor;
   for FListCounter := 0 to Items.Count - 1 do
    begin
     FTempPosLow := FTempPosHigh;
     FTempPosHigh := FTempPosHigh + FButtonHeight;
     if FBHDTempRemain > 0 then
      begin
       Inc(FTempPosHigh);
       Dec(FBHDTempRemain);
      end;
     FCaptionHeight := FBack.Canvas.TextHeight(Items[FListCounter]);
     FCaptionPosY := ((FTempPosLow + FTempPosHigh) div 2) - (FCaptionHeight div 2);
     if FButtonAlignment = baLeft then
      begin
       FCaptionPosX := FButtonWidth + (FMarginWidth div 2) - (FCaptionWidth div 2) + 1;
      end
     else if FButtonAlignment = baRight then
      begin
       FCaptionPosX := (FMarginWidth div 2) - (FCaptionWidth div 2) - 1;
      end;
     FBack.Canvas.TextOut(FCaptionPosX, FCaptionPosY, Items[FListCounter]);
    end;
  end;
 VisibleRect := Rect(0, 0, Width, Height);
 BackRect := Rect(0, 0, FBack.Width, FBack.Height);
 Canvas.CopyRect(VisibleRect, FBack.Canvas, BackRect);
end;

procedure TLCARSRadioGroup.Change;
begin
 if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TLCARSRadioGroup.SetDisabledColor(value : TColor);
begin
 if FDisabledColor <> value then
  begin
   FDisabledColor := value;
   Repaint;
  end;
end;

procedure TLCARSRadioGroup.SetEnabledColor(value : TColor);
begin
 if FEnabledColor <> value then
  begin
   FEnabledColor := value;
   Repaint;
  end;
end;

procedure TLCARSRadioGroup.SetMarginColor(value : TColor);
begin
 if FMarginColor <> value then
  begin
   FMarginColor := value;
   Repaint;
  end;
end;

procedure TLCARSRadioGroup.SetItemIndex(value : Integer);
begin
 if FItemIndex <> value then
  begin
   FItemIndex := value;
   Change;
   Repaint;
  end;
end;

procedure TLCARSRadioGroup.SetItems(Value : TStrings);
begin
 FItems.Assign(Value);
 Repaint;
end;

procedure TLCARSRadioGroup.SetButtonAlignment(Value : TButtonAlignment);
begin
 if FButtonAlignment <> value then
  begin
   FButtonAlignment := value;
   Repaint;
  end;
end;

procedure TLCARSRadioGroup.SetButtonShape(Value : TBMShape);
begin
 if FButtonShape <> value then
  begin
   FButtonShape := value;
   Repaint;
  end;
end;

procedure TLCARSRadioGroup.SetMarginShape(Value : TBMShape);
begin
 if FMarginShape <> value then
  begin
   FMarginShape := value;
   Repaint;
  end;
end;

procedure TLCARSRadioGroup.SetMarginStyle(Value : TMarginStyle);
begin
 if FMarginStyle <> value then
  begin
   FMarginStyle := value;
   Repaint;
  end;
end;

procedure TLCARSRadioGroup.SetButtonWidth(Value : Integer);
begin
 if FButtonWidth <> Value then
  begin
   FButtonWidth := Value;
   FMarginWidth := Width - Value;
   Repaint;
  end;
end;

procedure TLCARSRadioGroup.SetMarginRadius(Value : Integer);
begin
 if FMarginRadius <> Value then
  begin
   FMarginRadius := Value;
   Repaint;
  end;
end;

procedure TLCARSRadioGroup.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  if not FMouseInControl and Enabled {and IsForegroundTask } then begin
    FMouseInControl := True;
    MouseEnter;
  end;
end;

procedure TLCARSRadioGroup.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  if FMouseInControl and Enabled then begin
    FMouseInControl := False;
    MouseLeave;
  end;
end;

procedure TLCARSRadioGroup.MouseEnter;
begin
  if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
end;

procedure TLCARSRadioGroup.MouseLeave;
begin
  if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
end;

end.
