{----------------------------------------------------}
{ Andreis LCARS-Components 1.2 for Delphi 2, (3), 4 }
{----------------------------------------------------}
{ Copyright (c) 1999 Andrei Scheibner                }
{ http://www.htwm.de/~ascheibn                       }
{ ascheibn@htwm.de                                   }
{----------------------------------------------------}
{These Components are Freeware. You can distribute them}
{altogether, you can improve them , but I would appreciate}
{it, if my name appears somewhere in your improvements.}

unit LCARSCheckBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TCaptionPosition = (cpCenter, cpTopLeft, cpTopRight, cpBottomRight, cpBottomLeft);
  TLCARSCheckBox = class(TGraphicControl)
  private
   FCheckedColor : TColor;
   FUncheckedColor : TColor;
   FDisabledColor : TColor;
   FBack : TBitmap;
   FBackColor : TColor;
   FCaption : TCaption;
   FCaptionWidth : integer;
   FCaptionHeight : integer;
   FCaptionPosX : integer;
   FCaptionPosY : integer;
   FRadius : integer;
   FCaptionMarginX : integer;
   FCaptionMarginY : integer;
   FCaptionPosition : TCaptionPosition;
   FAlignment: TLeftRight;
   FState: TCheckBoxState;
   FAllowGrayed: Boolean;
   FHorSpace : integer;
   FRectWidth : integer;
   FMouseInControl: Boolean;
   FOnMouseEnter: TNotifyEvent;
   FOnMouseLeave: TNotifyEvent;
   procedure SetUncheckedColor(value : TColor);
   procedure SetCheckedColor(value : TColor);
   procedure SetDisabledColor(value : TColor);
   procedure SetCaption(value : TCaption);
   procedure SetCaptionPosition(value : TCaptionPosition);
   procedure SetCaptionMarginX(value : integer);
   procedure SetCaptionMarginY(value : integer);
   function GetChecked: Boolean;
   procedure SetState(Value: TCheckBoxState);
   procedure SetAlignment(Value: TLeftRight);
   procedure SetChecked(Value: Boolean);
   procedure CMEnabledChanged(var Msg: TMessage); message CM_ENABLEDCHANGED;
   procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
   procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
   procedure Paint; override;
   procedure Click; override;
   procedure MouseEnter; dynamic;
   procedure MouseLeave; dynamic;
  public
   constructor Create(AOwner : TComponent) ; override;
   destructor Destroy; override;
   procedure Toggle; virtual;
   property MouseInControl: Boolean read FMouseInControl;
  published
   property Anchors;
   property Constraints;
   property Parent;
   property Width default 129;
   property Height default 25;
   property Visible default true;
   property Enabled default true;
   property Font;
   property ParentFont default true;
   property Hint;
   property ShowHint;
   property ParentShowHint;
   property Align;
   property Alignment: TLeftRight read fAlignment write SetAlignment default taRightJustify;
   property AllowGrayed: Boolean read fAllowGrayed write fAllowGrayed default False;
   property State: TCheckBoxState read fState write SetState default cbUnchecked;
   property CaptionPosition : TCaptionPosition read FCaptionPosition
    write SetCaptionPosition default cpCenter;
   property Caption : TCaption read FCaption
    write SetCaption;
   property UncheckedColor : TColor read FUncheckedColor
    write SetUncheckedColor default clgreen;
   property CheckedColor : TColor read FCheckedColor
    write SetCheckedColor default cllime;
   property DisabledColor : TColor read FDisabledColor
    write SetDisabledColor default clgray;
   property CaptionMarginX : integer read FCaptionMarginX
    write SetCaptionMarginX default 2;
   property CaptionMarginY : integer read FCaptionMarginY
    write SetCaptionMarginY default 0;
   property Checked : boolean read GetChecked
    write SetChecked default false;
   property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
   property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
   property OnClick;
   property OnDblClick;
   property OnMouseDown;
   property OnMouseUp;
   property OnMouseMove;
   property OnDragDrop;
   property OnDragOver;
   property OnEndDrag;
  end;

procedure Register;

implementation

constructor TLCARSCheckBox.Create(AOwner : TComponent);
begin
 inherited Create(AOwner);
 Width := 129;
 Height := 25;
 UncheckedColor := clgreen;
 CheckedColor := cllime;
 DisabledColor := clgray;
 Visible := true;
 Alignment := taRightJustify;
 Checked := false;
 AllowGrayed := false;
 CaptionPosition := cpCenter;
 FRadius := Height div 2;
 FHorSpace := 8;
 Canvas.Brush.Style := bsClear;
 ControlStyle:=ControlStyle+[csOpaque];
 FBack := TBitmap.Create;
 CaptionMarginX := 2;
 CaptionMarginY := 0;
 Caption := Name;
 ParentFont := true;
end;

destructor TLCARSCheckBox.Destroy;
begin
 FBack.Free;
 inherited Destroy;
end;

procedure TLCARSCheckBox.Click;
begin
 Toggle;
 inherited Click;
end;

procedure TLCARSCheckBox.Paint;
var VisibleRect, BackRect : TRect;
begin
 FBack.Canvas.Font.Assign(Font);
 FBack.Width := Width;
 FBack.Height := Height;
 if Parent <> nil then
  with Parent do
   FBackColor := Color;
 FRadius := Height div 2;
 FRectWidth := Width - FRadius - FHorSpace;
 FCaptionWidth := FBack.Canvas.TextWidth(FCaption);
 FCaptionHeight := FBack.Canvas.TextHeight(FCaption);
 FBack.Canvas.Brush.Color := FBackColor;
 FBack.Canvas.Pen.Color := FBackColor;
 FBack.Canvas.Rectangle(0, 0, FBack.Width, FBack.Height);
 if Enabled then
  begin
   if Checked then
    begin
     FBack.Canvas.Brush.Color := CheckedColor;
     FBack.Canvas.Pen.Color := CheckedColor;
    end
   else if not Checked then
    begin
     FBack.Canvas.Brush.Color := UncheckedColor;
     FBack.Canvas.Pen.Color := UncheckedColor;
    end;
  end
 else
  begin
   FBack.Canvas.Brush.Color := DisabledColor;
   FBack.Canvas.Pen.Color := DisabledColor;
  end;
 if Alignment = taRightJustify then
  begin
   FBack.Canvas.MoveTo(FRadius, 0);
   FBack.Canvas.LineTo(FRadius, FBack.Height);
   FBack.Canvas.Arc(0, 0, FBack.Height, FBack.Height,
                    FRadius, 0, FRadius, FBack.Height);
   FBack.Canvas.FloodFill(FRadius - 2, FRadius,
                          FBack.Canvas.Pixels[FRadius - 2,
                          FRadius], fssurface);
   FBack.Canvas.Rectangle(FRadius, 0,
                          FRadius + 4, FBack.Height);
   FBack.Canvas.Rectangle(FRadius + FHorSpace, 0,
                          FBack.Width, FBack.Height);
  end
 else if Alignment = taLeftJustify then
  begin
   FBack.Canvas.MoveTo(FBack.Width - FRadius, 0);
   FBack.Canvas.LineTo(FBack.Width - FRadius, FBack.Height);
   FBack.Canvas.Arc(FBack.Width - FBack.Height, 0, FBack.Width ,FBack.Height,
                    FBack.Width - FRadius, FBack.Height,
                    FBack.Width - FRadius, 0);
   FBack.Canvas.FloodFill(FBack.Width - 2, FRadius,
                          FBack.Canvas.Pixels[FBack.Width - 2,
                          FRadius], fssurface);
   FBack.Canvas.Rectangle(FBack.Width - FRadius - 4, 0,
                          FBack.Width - FRadius, FBack.Height);
   FBack.Canvas.Rectangle(0, 0, FRectWidth, FBack.Height);
  end;
 case FCaptionPosition of
  cpCenter :
   begin
    if Alignment = taRightJustify then
     begin
      FCaptionPosX := FRadius + FHorSpace + (FRectWidth div 2) - (FCaptionWidth div 2);
      FCaptionPosY := (FBack.Height div 2) - (FCaptionHeight div 2);
     end
    else if Alignment = taLeftJustify then
     begin
      FCaptionPosX := (FRectWidth div 2) - (FCaptionWidth div 2);
      FCaptionPosY := (FBack.Height div 2) - (FCaptionHeight div 2);
     end;
   end;
  cpTopLeft :
   begin
    if Alignment = taRightJustify then
     begin
      FCaptionPosX := FRadius + FHorSpace + CaptionMarginX;
      FCaptionPosY := CaptionMarginY;
     end
    else if Alignment = taLeftJustify then
     begin
      FCaptionPosX := CaptionMarginX;
      FCaptionPosY := CaptionMarginY;
     end;
   end;
  cpTopRight :
   begin
    if Alignment = taRightJustify then
     begin
      FCaptionPosX := FBack.Width - CaptionMarginX - FCaptionWidth;
      FCaptionPosY := CaptionMarginY;
     end
    else if Alignment = taLeftJustify then
     begin
      FCaptionPosX := FRectWidth - CaptionMarginX - FCaptionWidth;
      FCaptionPosY := CaptionMarginY;
     end;
   end;
  cpBottomRight :
   begin
    if Alignment = taRightJustify then
     begin
      FCaptionPosX := FBack.Width - CaptionMarginX - FCaptionWidth;
      FCaptionPosY := FBack.Height - CaptionMarginY - FCaptionHeight;
     end
    else if Alignment = taLeftJustify then
     begin
      FCaptionPosX := FRectWidth - CaptionMarginX - FCaptionWidth;
      FCaptionPosY := FBack.Height - CaptionMarginY - FCaptionHeight;
     end;
   end;
  cpBottomLeft :
   begin
    if Alignment = taRightJustify then
     begin
      FCaptionPosX := FRAdius + FHorSpace + CaptionMarginX;
      FCaptionPosY := FBack.Height - CaptionMarginY - FCaptionHeight;
     end
    else if Alignment = taLeftJustify then
     begin
      FCaptionPosX := CaptionMarginX;
      FCaptionPosY := FBack.Height - CaptionMarginY - FCaptionHeight;
     end;
   end;
 end;
 FBack.Canvas.TextOut(FCaptionPosX, FCaptionPosY, FCaption);
 VisibleRect := Rect(0, 0, Width, Height);
 BackRect := Rect(0, 0, FBack.Width, Fback.Height);
 Canvas.CopyRect(VisibleRect, FBack.Canvas, BackRect);
end;

procedure TLCARSCheckBox.SetAlignment(Value: TLeftRight);
begin
 if Value <> FAlignment then
  begin
   FAlignment := Value;
   Repaint;
  end;
end;

procedure TLCARSCheckBox.Toggle;
begin
 case State of
  cbUnchecked: if AllowGrayed then
                State := cbGrayed
               else
                State := cbChecked;
  cbChecked: State := cbUnchecked;
  cbGrayed: State := cbChecked;
 end;
end;

procedure TLCARSCheckBox.SetState(Value: TCheckBoxState);
begin
 if fState <> Value then
  begin
   fState := Value;
   Repaint;
  end;
end;

procedure TLCARSCheckBox.SetChecked(Value: Boolean);
begin
 if Enabled then
  begin
   if Value then
    State:=cbChecked
   else
    State:=cbUnchecked;
  end;
end;

procedure TLCARSCheckBox.CMEnabledChanged(var Msg: TMessage);
begin
     inherited;
     Repaint;
end;

function TLCARSCheckBox.GetChecked: Boolean;
begin
 Result := (State = cbChecked);
end;

procedure TLCARSCheckBox.SetCaption(value : TCaption);
begin
 if FCaption <> value then
  begin
   FCaption := value;
   Refresh;
  end;
end;

procedure TLCARSCheckBox.SetCaptionPosition(value : TCaptionPosition);
begin
 if FCaptionPosition <> value then
  begin
   FCaptionPosition := value;
   Repaint;
  end;
end;

procedure TLCARSCheckBox.SetUncheckedColor(value : TColor);
begin
 if FUncheckedColor <> value then
  begin
   FUncheckedColor := value;
   Repaint;
  end;
end;

procedure TLCARSCheckBox.SetCheckedColor(value : TColor);
begin
 if FCheckedColor <> value then
  begin
   FCheckedColor := value;
   Repaint;
  end;
end;

procedure TLCARSCheckBox.SetDisabledColor(value : TColor);
begin
 if FDisabledColor <> value then
  begin
   FDisabledColor := value;
   Repaint;
  end;
end;

procedure TLCARSCheckBox.SetCaptionMarginX(value : integer);
begin
 if FCaptionMarginX <> value then
  begin
   FCaptionMarginX := value;
   Repaint;
  end;
end;

procedure TLCARSCheckBox.SetCaptionMarginY(value : integer);
begin
 if FCaptionMarginY <> value then
  begin
   FCaptionMarginY := value;
   Repaint;
  end;
end;

procedure TLCARSCheckBox.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  if not FMouseInControl and Enabled {and IsForegroundTask } then begin
    FMouseInControl := True;
    MouseEnter;
  end;
end;

procedure TLCARSCheckBox.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  if FMouseInControl and Enabled then begin
    FMouseInControl := False;
    MouseLeave;
  end;
end;

procedure TLCARSCheckBox.MouseEnter;
begin
  if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
end;

procedure TLCARSCheckBox.MouseLeave;
begin
  if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
end;

procedure Register;
begin
  RegisterComponents('LCARS-Comps', [TLCARSCheckBox]);
end;

end.